*****************************************************
* Regression in Stata - QIPSR
* Do file created by: Matthew Cain							*
*****************************************************

*QIPSR Facebook: https://www.facebook.com/QIPSR*
*QIPSR website: http://qipsr.as.uky.edu/*


version 15
clear all

**************
* The Basics *
**************

	* Importing some data *
	sysuse auto, clear

	* Running a simple linear regression *
	reg price mpg

	* Running a multivariate regression *
	reg price mpg weight foreign
	
	
****************
* Interactions *
****************

	* Adding an interaction between two continuous variables *
	reg price c.mpg##c.weight foreign

	* Adding an interaction between a continuous and a categorical variable
	reg price mpg c.weight##i.foreign

	
	
******************
* Postestimation *
******************
	reg price mpg weight foreign
	
	* Goodness of Model Fit - Lower AIC/BIC Indicates Better Model Fit *
	estat ic

	* Predicted Values *
	predict y

	* Residuals *
	predict e, r
	
	
	* Comparing two regression coefficients *
	test mpg = weight

	
	
**************************
* Regression Diagnostics *
**************************

	* Checking Normality of Residuals *
	swilk e
	kdensity e, normal
	
	
	* Checking for Linearity *
	scatter price mpg
	lowess price mpg
	lowess price weight
	acprplot mpg, lowess
	acprplot weight, lowess


	* Checking for Unusual/Influential Data *
	gen obs=_n

		*Standardized and Studentize Residuals - For Identifying Outliers *
		predict stan, rstan
		predict stud, rstud
		* Rule of Thumb - Visibly distant observations or values greater than 2 may be problematic *
		scatter stan obs, mlabel(obs) yline(-2 2)
		scatter stud obs, mlabel(obs) yline(-2 2)

		* Hat Values - For Identifying High-Leverage Observations *
		predict lev, hat
		* Rule of Thumb - Visibly distant observations may be problematic *
		scatter lev obs, mlabel(obs)

		* Cooks-D and DFITS - For Identifying High-Influence Observations *
		predict d, cooksd
		predict dfits, dfits
		
		* Visibly distant observations or observations greater than 1 may be problematic *
		scatter d obs, mlabel(obs) yline(1)
		* Visibly distant observations may be problematic
		scatter dfits obs, mlabel(obs)


	* Checking for Heteroskedasticity *
	reg price mpg weight foreign

		* Breusch-Pagan Test - Significance is evidence of heteroskedasticity *
		* We can test either the whole model or individual variables *
		estat hettest
		estat hettest mpg
		estat hettest weight
		estat hettest foreign

		* Szroeter's Test - Significance is evidence heteroskedasticity *
		* We can test each variable in the model with the "rhs" option *
		estat szroeter, rhs

		* Residuals vs Fitted values plot - Any discernible pattern can be evidence of heteroskedasticity *
		rvfplot
		


	* Checking for Multicolinearity *
		* Correlation Matrix of the Independent Variables - Good for a first pass *
		pwcorr mpg weight foreign
		
		* Scatter plots of all independent variables - Look for linear relationships *
		graph matrix mpg weight foreign, half
		
		* Variance Inflation Factor - The lower the VIF the better *
		estat vif	


	* Omitted Variable Bias *
	estat ovtest


	* Checking for Serial Autocorrelation *
	webuse hardware, clear

		* Use tsset to identify units of time *
		tsset qdate

		reg unrate gdp
		
		* Graphical test - we want to see no relationship between the error term and time *
		predict e, r
		scatter e qdate, yline(0)

		* The Durbin-Watson Statistic - We want this to be as close to 2 as possible *
		estat dwatson

		* Breush-Godfrey Test - Significance is evidence of autocorrelation *
		estat bgodfrey

		* Durbin's Alternative Test - Significance is evidence of autocorrelation *
		estat durbinalt

		* We can have stata test for higher-order autocorrelation with the lag() option *
		estat bgodfrey, lag(1/5)
		estat durbinalt, lag(1/5)


	*Checking for Spatial Autocorrelation is more complicated - see spset, spmatrix, and spregress *


	
******************************************************
* Adjusting for Violations of Regression Assumptions *
******************************************************
	sysuse auto, clear
	
	* Heteroskedasticity *
		* Robust Standard Errors *
		reg price mpg weight foreign, robust

		* Clustered Standard Errors *
		reg price mpg weight foreign, cluster(foreign)
		
	
	* Other Variations on Standard Errors *
		* Bootstrapped Standard Errors *
		bootstrap: reg price mpg weight foreign
		bootstrap, reps(500): reg price mpg weight foreign
		
		* Jackknifed Standard Errors *
		jackknife: reg price mpg weight foreign


	* Serial Autocorrelation *
	webuse hardware, clear
	tsset qdate

		* Lagged Dependent Variable *
		reg unrate gdp L.unrate
		reg unrate gdp L1.unrate L2.unrate
		reg unrate gdp L(1/5).unrate

		* First-Differences *
		reg D.unrate D.gdp


	
	
***********************
* Other Common Models *
***********************

* Probability Models *
sysuse auto, clear

* Logistic Regression *
logit foreign price weight

* Probit Regression *
probit foreign price weight

* Complementary Log-Log Regression *
cloglog foreign price weight

* Exact Logistic Regression - Useful for small samples *
exlogistic foreign rep78, memory(2g)
* See also firthlogit and relogit *



* Ordered Models *

* Ordered Logit *
ologit rep78 price weight

* Ordered Probit *
oprobit rep78 price weight



* Multinomial Models *
use https://stats.idre.ucla.edu/stat/data/hsbdemo, clear

* Multinomial Logit *
mlogit prog write math
mlogit prog write math, b(1)

* Multinomial Probit *
mprobit prog write math



*Count Models*
webuse dollhill3, clear

* Poisson Regression *
poisson deaths smokes

* Negative-Binomial Regression *
nbreg deaths smokes



* Truncated/Censored Models *

* Tobit Regression *
use https://stats.idre.ucla.edu/stat/stata/dae/tobit, clear
* The dependent variable is right-censored at 800 *
tobit apt read math, ul(800)

* Truncated Regression
use https://stats.idre.ucla.edu/stat/stata/dae/truncreg, clear
* The dependent variable has a lower limit of 40 *
truncreg achiv langscore, ll(40)



* Selection Models *

* Heckman Selection Model *
webuse womenwk, clear

heckman wage education, select(married children)
* heckprobit and heckoprobit use the same syntax as heckman *



* Duration/Survival Models * 
webuse drugtr, clear
stset

* Generate Kaplan-Meier Graph *
sts

* Non-Parametric - Cox Regression *
stcox drug age

* Parametric - Specify Distribution using "dist()" *
streg drug age, dist(lnorm)



* The GLM Command *
sysuse auto, clear

* Example - Linear Regression *
glm price mpg weight, family(gaussian) link(identity)

* Example - Probit Regression *
glm foreign mpg weight, family(binomial) link(probit)
